Package: SparkR
Type: Package
Version: 3.1.2
Title: R Front End for 'Apache Spark'
Description: Provides an R Front end for 'Apache Spark' <https://spark.apache.org>.
Authors@R: c(person("Shivaram", "Venkataraman", role = "aut",
                    email = "shivaram@cs.berkeley.edu"),
             person("Xiangrui", "Meng", role = "aut",
                    email = "meng@databricks.com"),
             person("Felix", "Cheung", role = c("aut", "cre"),
                    email = "felixcheung@apache.org"),
             person(family = "The Apache Software Foundation", role = c("aut", "cph")))
License: Apache License (== 2.0)
URL: https://www.apache.org https://spark.apache.org
BugReports: https://spark.apache.org/contributing.html
SystemRequirements: Java (>= 8, < 12)
Depends: R (>= 3.5), methods
Suggests: knitr, rmarkdown, markdown, testthat, e1071, survival, arrow
        (>= 1.0.0)
Collate: 'schema.R' 'generics.R' 'jobj.R' 'column.R' 'group.R' 'RDD.R'
        'pairRDD.R' 'DataFrame.R' 'SQLContext.R' 'WindowSpec.R'
        'backend.R' 'broadcast.R' 'catalog.R' 'client.R' 'context.R'
        'deserialize.R' 'functions.R' 'install.R' 'jvm.R'
        'mllib_classification.R' 'mllib_clustering.R' 'mllib_fpm.R'
        'mllib_recommendation.R' 'mllib_regression.R' 'mllib_stat.R'
        'mllib_tree.R' 'mllib_utils.R' 'serialize.R' 'sparkR.R'
        'stats.R' 'streaming.R' 'types.R' 'utils.R' 'window.R'
RoxygenNote: 7.1.1
VignetteBuilder: knitr
NeedsCompilation: no
Encoding: UTF-8
Packaged: 2021-05-24 04:42:43 UTC; spark-rm
Author: Shivaram Venkataraman [aut],
  Xiangrui Meng [aut],
  Felix Cheung [aut, cre],
  The Apache Software Foundation [aut, cph]
Maintainer: Felix Cheung <felixcheung@apache.org>
Built: R 4.1.0; ; 2021-05-24 04:45:04 UTC; unix
